---------------------------------------------------------------------------------------------------
--
-- Title       : periferie
-- Author      : Rupert Ploner
-- Company     : Informatik
--
---------------------------------------------------------------------------------------------------
--
-- File        : periferie.vhd
-- Generated   : Tue May 25 14:24:03 2004
--
---------------------------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_unsigned.all;
use ieee.numeric_std.all;
-- synopsys translate_off 
library SPARTAN2E;
library IEEE;
use IEEE.vital_timing.all;
-- synopsys translate_on 

entity periferie is
	port(
		WR : in STD_LOGIC;							 --CPU WR
		RD : in STD_LOGIC;							 --CPU RD
		DIR : in STD_LOGIC;							 --CPU DIR
		CLK : in STD_LOGIC;	   						 --Clock
		RST : in STD_LOGIC;							 --Reset
		ADDR : in STD_LOGIC_VECTOR(15 downto 0);	 --CPU ADDR
		DATA : inout STD_LOGIC_VECTOR(15 downto 0);	 --CPU Data
		WEn_sram : out STD_LOGIC;					 -- fuer sram
		BHEn_sram : out STD_LOGIC;
		BLEn_sram : out STD_LOGIC;
		OEn_sram : out STD_LOGIC;
		CSn_sram : out STD_LOGIC;
		RWn_cpld : out STD_LOGIC;					 -- R/W# cpld
		CSn_cpld : out STD_LOGIC;					 -- CS# cpld
		ext_rw : out STD_LOGIC;						 -- rw LCD
		CSn_lcd : out STD_LOGIC;					 -- CS# LCD
		CS_lcd : out STD_LOGIC;						 -- CS LCD
		READY : out STD_LOGIC;						 -- CPU Ready
		ext_a : out STD_LOGIC_VECTOR(2 downto 0);	 -- LCD ADressbus
		ext_d : out STD_LOGIC_VECTOR(7 downto 0);	 -- LCD Data
		mem_a : out STD_LOGIC_VECTOR(21 downto 1);	 -- memory subsystem addr
		mem_d : inout STD_LOGIC_VECTOR(15 downto 0); -- memory subsystem data
		RS232in : in STD_LOGIC;						 -- RS232 in
		RS232out : out STD_LOGIC					 -- RS232 out
		);
end periferie;


architecture periferie of periferie is
	
	signal DATAin, DATAout,DATAmem : std_logic_vector(15 downto 0);
	signal line1,line2: String(1 to 16);
	signal div: std_logic_vector (14 downto 0);
	signal loc_clk : STD_LOGIC;	
	
	signal reset: std_logic;								-- lcd reset sequence active
	signal cmd: std_logic_vector (7 downto 0);				-- byte written to lcd
	signal rs: std_logic;									-- register select bit for lcd
	signal idx: integer range 0 to 3;						-- sequencer index for lcd timing
	signal cs:  std_logic;									-- chip select for lcd
	signal cnt: std_logic_vector (5 downto 0);				-- character, command counter
	
	
	signal pb : std_logic_vector(3 downto 0);				-- push buttons	
	signal led : std_logic_vector(3 downto 0);				-- leds
	signal dip : std_logic_vector(7 downto 0);				-- dip switches
	signal din : std_logic_vector(15 downto 0);
	signal dout : std_logic_vector(15 downto 0);
	signal ddir : std_logic;
	signal state	: std_logic_vector(4 downto 0); 		-- state counter for sequenzer
	
	signal en_mem: std_logic;								-- Block ram
	signal ad_mem: std_logic_vector(7 downto 0);
	signal we: std_logic;
	signal memclk: std_logic;                               -- clock for BlockRAM 
	
	
	
	component RAMB4_S16
		generic (
			INIT_00:bit_vector;	
			INIT_01:bit_vector;
			INIT_02:bit_vector;
			INIT_03:bit_vector;
			INIT_04:bit_vector; 
			INIT_05:bit_vector;
			INIT_06:bit_vector;
			INIT_07:bit_vector;
			INIT_08:bit_vector;
			INIT_09:bit_vector;
			INIT_0A:bit_vector;
			INIT_0B:bit_vector;
			INIT_0C:bit_vector;
			INIT_0D:bit_vector;
			INIT_0E:bit_vector;
			INIT_0F:bit_vector
			);
		port (DI     : in STD_LOGIC_VECTOR (15 downto 0);
			EN     : in STD_ULOGIC;
			WE     : in STD_ULOGIC;
			RST    : in STD_ULOGIC;
			CLK    : in STD_ULOGIC;
			ADDR   : in STD_LOGIC_VECTOR (7 downto 0);
			DO     : out STD_LOGIC_VECTOR (15 downto 0)
			);
	end component;
	
	-- synopsys translate_off
	for U1 : RAMB4_S16 use entity spartan2e.RAMB4_S16;
	-- synopsys translate_on
	
begin 
	
	DATA<=DATAout when dir='0' else (others=>'Z');--Datenbustreiber
	DATAIN<=DATA;							 
	
	U1: RAMB4_S16
	generic map (
	INIT_0F=>x"0000000000000000000000000000000000000000000000000000000000000000", 
	INIT_0E=>x"0000000000000000000000000000000000000000000000000000000000000000",	
	INIT_0D=>x"0000000000000000000000000000000000000000000000000000000000000000",	
	INIT_0C=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_0B=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_0A=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_09=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_08=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_07=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_06=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_05=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_04=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_03=>x"0000000000000000000000000000000000000000000000000000000000000000", 
	INIT_02=>x"0000000000000000000000000000000000000000000000000000000000000000",
	INIT_01=>
		"0000000000000000"&-- 1F: 
		"0000000000000000"&-- 1E: 
		"0000000000000000"&-- 1D: 
		"0000000000000000"&-- 1C: 
		"0000000000000000"&-- 1B: 
		"0000000000000000"&-- 1A: 
		"0000000000000000"&-- 19: 
		"0000000000000000"&-- 18: 
		"0000000000000000"&-- 17: 
		"0000000000000000"&-- 16: 
		"0000000000000000"&-- 15: 
		"0000000000000000"&-- 14: 
		"0000000000000000"&-- 13: 
		"0000000000000000"&-- 12: 
		"0000000000000000"&-- 11: 
		"0000000000000000",-- 10: 
	INIT_00=>	
		"0000000000000000"& --0F: 
		"0000000000000000"& --0E: 
		"0000000000000000"& --0D: 
		"0000000000000000"& --0C:  
		"0000000000000000"& --0B:  
		"0000000000000000"& --0A:  
		"0000000000000000"& --09:  
		"0000000000000000"& --08:  
		"0000000000000000"& --07:  
		"0000000000000000"& --06:  
		"0000000000000000"& --05:  
		"0000000000000000"& --04:  
		"0000000000000000"& --03:  
		"0000000000000000"& --02:  
		"1010000011100000"& --01:  ldil r0,#xE0
		"1000000011111111"  --00:  ldih r0,#xFF	   	  Programm start
	)										

	
-- Blockram	
	port map	(				
		datain, EN_mem,we, RST, memclk, ad_mem, datamem
		);
	
-- clock for BlockRAM 
	memclk <= not clk;
	
-- eigentlicher lese und schreibprozess	
	operate: process(clk,RST,RD,WR) 
		
	begin 		
		if RST='1' then
			ready<='1';
			we<='0'; 
			ad_mem<=(others =>'0');
			en_mem<='1';
			line1<= (others=>' ');
			line2<= (others=>' ');
			led<=(others =>'0');
			DATAout<=(others=>'0');
			
		elsif clk'event and clk='1' then
			
			if RD='1' then 							 -- lesen
				if ADDR>=x"FFF0" then				 -- Zeile 2
					DATAout<= std_logic_vector(to_unsigned(character'pos(line2(conv_integer(ADDR(3 downto 0)))), 16));
				elsif ADDR>=x"FFE0" then			 -- Zeile 1
					DATAout<= std_logic_vector(to_unsigned(character'pos(line1(conv_integer(ADDR(3 downto 0)))), 16));
				elsif ADDR=x"FFDF" then				 -- dips, pushbuttons und leds lesen
					DATAout<=dip & pb & led;						   
				elsif ADDR>=x"0100" then
					DATAout<=x"0000";				 -- default 0, wo kein Speicher eingeblendet
				else								 -- sonst vom blockram
					ad_mem<=ADDR(7 downto 0);
					DATAout<=DATAmem;
					EN_mem<='1';	
				end if;
				ready<='1';		
			elsif WR='1' and we='0'then						 -- schreiben
				we<='1';
				if ADDR>=x"FFF0" then 						 -- Zeile 2
					line2(1+conv_integer(ADDR(3 downto 0)))<=character'Val(conv_integer(DATAin(7 downto 0)));	
				elsif ADDR>=x"FFE0" then 					 -- Zeile 1
					line1(1+conv_integer(ADDR(3 downto 0)))<=character'Val(conv_integer(DATAin(7 downto 0)));
				elsif ADDR=x"FFDF" then						 -- leds schreiben
					led<=DATAin(3 downto 0);
				elsif ADDR>=x"0100" then
					null;
				else										 -- sonst ins ram
					ad_mem<=ADDR(7 downto 0);
					EN_mem<='1';
				end if;
			elsif WR='1' and we='1' then
				ready<='1';
				we<='0';
			else  
				we<='0';	
				en_mem<='0';
				ready<='0';
			end if;
		end if;
	end process; 				   
	
	-----------------------------------------------------------------
	------------------------------ dips pb leds schreiben und lesen
	
	mem_d<=dout when ddIR='0' else (others=>'Z');--Datenbustreiber
	din<=mem_d;
	
	process(clk, rst)
	begin
		if rst= '1' then
			state<= (others=> '0');	 						-- reset state counter
		elsif rising_edge(clk) then
			state<= state + 1;								-- advance state on every clock
		end if;
	end process;															 
	
	process(clk, rst)
	begin
		if rst= '1' then
			CSn_cpld <= '1';
			RWn_cpld <= '1';		
			ddir<='1';
			mem_a(2 downto 1)	<= "00";
			pb<=(others=>'0');
			dip<=(others=>'0');
			dout<=(others=>'0');
		elsif rising_edge(clk) then
			case state is
				when "00000"	=>	mem_a(2 downto 1)<= "00"; -- address = 00 => push buttons
				CSn_cpld <= '1';
				RWn_cpld <= '1';			-- read from cpld
				ddir<='1';
				when "00001"  =>	CSn_cpld <= '0';			-- chip select for cpld active
				when "00010"  =>	pb <= din(3 downto 0);	    -- latch cpld data
				when "00011" =>     CSn_cpld <= '1';			-- chip select inactive
				when "00100"  =>	mem_a(2 downto 1) <="01";	-- address = 01 => dip switches
				when "00101"  =>	CSn_cpld <= '0';			-- chip select for cpld active
				when "00110"  =>	dip <= din(7 downto 0);	    -- latch cpld data
				when "00111"  =>	CSn_cpld <= '1';			-- chip select inactive
				when "01000"  =>	mem_a(2 downto 1) <= "10";	-- address = 10 => led's
				RWn_cpld <= '0';			-- write to cpld	
				ddir<='0';
				when "01001"  =>	dout<="000000000000" & led;	-- put data on bus
				when "01010"  =>	CSn_cpld <= '0';			-- generate chip select pulse
				when "01011"  =>	CSn_cpld <= '1';
				when others   =>    CSn_cpld<='1';              --pause	
				ddir<='1';
			end case;	
		end if;
	end process;
	
	
	----------------------------------------------------
	----------------------LCD-Panel ansteuern
	
	-- clock & reset
	process(clk, rst)
	begin
		if rst= '1' then
			div <= (others=> '0');
		elsif rising_edge(clk) then
			div<= div + 1;
		end if;
	end process;
	
	-- frequenzteiler
	loc_clk<= div(div'left);
	
	-- sequencer
	process(loc_clk, rst)
	begin
		if rst= '1' then 
			idx <= 0;
		elsif rising_edge(loc_clk) then
			if idx= 3 then idx<= 0; else idx<= idx + 1; end if;
		end if;
	end process;
	
	------------------------------------
	-- lcd signals
	
	process(loc_clk, rst)
	begin
		if rst= '1' then
			cs    <= '0';
			ext_a <= (others=> '0');
			ext_d <= (others=> '0');
		elsif rising_edge(loc_clk) then
			case idx is
				when 0=> ext_a<= "00" & rs;					-- set register adress
				when 1=> cs   <= '1';ext_d<= cmd;			-- write data, activate chip select
				when 2=> cs   <= '0';						-- deactivate chip select 
				when 3=> null;								-- data hold time
			end case;
		end if;
	end process;
	
	ext_rw<= '0';											-- always write
	cs_lcd <= cs;											-- assign external signals
	csn_lcd<= '0';											-- Data direction always -> LCD
	
	
	
	process(loc_clk, rst)
		-- write line1, line2 to LCD
	begin
		if rst= '1' then
			cmd	<= x"00";
			rs		<= '1';
			reset	<= '1';									-- activate LCD init at reset
			cnt <= (others => '0');
		elsif rising_edge(loc_clk) then
			if (reset ='0') then							-- if not LCD init						
				if idx = 2 then	cnt <= cnt + 1;
				end if;
				case cnt is
					when "000000" =>  cmd <= x"02";    rs <= '0';	-- cursor home
					when "000001" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(1)), 8)); rs <= '1';
					when "000010" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(2)), 8)); rs <= '1';
					when "000011" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(3)), 8)); rs <= '1';
					when "000100" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(4)), 8)); rs <= '1';
					when "000101" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(5)), 8)); rs <= '1';
					when "000110" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(6)), 8)); rs <= '1';
					when "000111" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(7)), 8)); rs <= '1';
					when "001000" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(8)), 8)); rs <= '1';
					when "001001" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(9)), 8)); rs <= '1';
					when "001010" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(10)), 8)); rs <= '1';
					when "001011" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(11)), 8)); rs <= '1';
					when "001100" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(12)), 8)); rs <= '1';
					when "001101" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(13)), 8)); rs <= '1';
					when "001110" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(14)), 8)); rs <= '1';
					when "001111" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(15)), 8)); rs <= '1';
					when "010000" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line1(16)), 8)); rs <= '1';
					when "010001" =>  cmd <= x"C0";    rs <= '0';	-- second line
					when "010010" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(1)), 8)); rs <= '1';
					when "010011" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(2)), 8)); rs <= '1';
					when "010100" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(3)), 8)); rs <= '1';
					when "010101" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(4)), 8)); rs <= '1';
					when "010110" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(5)), 8)); rs <= '1';
					when "010111" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(6)), 8)); rs <= '1';
					when "011000" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(7)), 8)); rs <= '1';
					when "011001" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(8)), 8)); rs <= '1';
					when "011010" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(9)), 8)); rs <= '1';
					when "011011" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(10)), 8)); rs <= '1';
					when "011100" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(11)), 8)); rs <= '1';
					when "011101" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(12)), 8)); rs <= '1';
					when "011110" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(13)), 8)); rs <= '1';
					when "011111" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(14)), 8)); rs <= '1';
					when "100000" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(15)), 8)); rs <= '1';
					when "100001" =>  cmd <= std_logic_vector(to_unsigned(character'pos(line2(16)), 8)); rs <= '1';
					when others   =>  cmd <= x"02"; rs <= '0';cnt <= "000000";
				end case;
			else												-- if LCD init
				if idx = 2 then	cnt <= cnt + 1;
				end if;
				case cnt is
					when "000000" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
					-- wait (see LCD Datasheet for details)
					when "100100" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
					when "101000" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
					when "101100" => cmd <= x"38"; rs <= '0';	-- Function Set   (8-bit, Dual Line)
					when "110000" => cmd <= x"0c"; rs <= '0';	-- Display On/Off (display on, cursor off, cursor blink off)
					when "110100" => cmd <= x"06"; rs <= '0';	-- Entry Mode Set (increment on, display shift off)
					when "111000" => cmd <= x"01"; rs <= '0';	-- Display Clear
					when "111100" => cmd <= x"02"; rs <= '0';	-- Display Cursor Home 
					when "111111" => reset <= '0'; cnt <= "000000";
					when others   => null;
				end case;
			end if;		
		end if;
	end process;
	
end periferie;
